#include "app_util_platform.h"
#include "app_timer.h"
#include "nrf_drv_gpiote.h"
#include "nrf_drv_twi.h"
#include "nrf_delay.h"
#include "eMPL_util.h"

static const nrf_drv_twi_t p_twi_instance = NRF_DRV_TWI_INSTANCE(1);
static nrf_drv_gpiote_pin_t _pin;

void nrf5x_twi_init()
{
	ret_code_t err_code;
	
	nrf_drv_twi_config_t    p_twi_config = NRF_DRV_TWI_DEFAULT_CONFIG(1); // Set up TWI configuration default values

	err_code = nrf_drv_twi_init(&p_twi_instance, &p_twi_config, NULL, NULL);
	APP_ERROR_CHECK(err_code);
}

void nrf5x_twi_enable()
{
	nrf_drv_twi_enable(&p_twi_instance);
}

void nrf5x_twi_disable()
{
	nrf_drv_twi_disable(&p_twi_instance);
}

/**
	Quick and dirty, should be called at least once per period, no checking done.
*/
int nrf5x_get_ms(unsigned long *count) 
{
	if (count == NULL) {
		return 1;
	}
	
	uint32_t current_tick_count;
	uint32_t tick_diff;
	static uint32_t last_tick_count = 0;
	static uint32_t ms_count = 0;
	
	app_timer_cnt_get(&current_tick_count);
	app_timer_cnt_diff_compute(current_tick_count, last_tick_count, &tick_diff);
	last_tick_count = current_tick_count;
	
	ms_count += (tick_diff * 1000) / APP_TIMER_CLOCK_FREQ;

	*count = ms_count;
	
	return 0;
}

int nrf5x_i2c_write(unsigned char slave_addr,
					   unsigned char reg_addr,
					   unsigned char length,
                       unsigned char const *data) 
{
	ret_code_t err_code;

	err_code = nrf_drv_twi_tx(&p_twi_instance, slave_addr, &reg_addr, sizeof(reg_addr), true);
	if (err_code == NRF_SUCCESS) {
		err_code = nrf_drv_twi_tx(&p_twi_instance, slave_addr, data, length, false);
	}
    
    return err_code;
}

int nrf5x_i2c_read(unsigned char slave_addr,
                    unsigned char reg_addr,
                    unsigned char length,
                    unsigned char *data)
{
	ret_code_t err_code;

	err_code = nrf_drv_twi_tx(&p_twi_instance, slave_addr, &reg_addr, sizeof(reg_addr), true);
	if (err_code == NRF_SUCCESS) {
		err_code = nrf_drv_twi_rx(&p_twi_instance, slave_addr, data, length);
	}
    
    return err_code;
}

void nrf5x_int_enable()
{
	nrf_drv_gpiote_in_event_enable(_pin, true);
}

void nrf5x_int_disable()
{
	nrf_drv_gpiote_in_event_enable(_pin, false);
}

int nrf5x_reg_int_cb(void (*cb)(void), unsigned short pin, unsigned char lp_exit, unsigned char active_low)
{
	ret_code_t err_code;
	
	_pin = pin;
	nrf_drv_gpiote_in_config_t _config  = GPIOTE_CONFIG_IN_SENSE_HITOLO(false);

	_config.sense = active_low ? NRF_GPIOTE_POLARITY_HITOLO : NRF_GPIOTE_POLARITY_LOTOHI;
	
	err_code = nrf_drv_gpiote_in_init(_pin, &_config, (nrf_drv_gpiote_evt_handler_t)cb);
	
	return err_code;
}
