/*
 * Copyright (c) 2014 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is confidential property of Nordic Semiconductor. The use,
 * copying, transfer or disclosure of such information is prohibited except by express written
 * agreement with Nordic Semiconductor.
 *
 */

#include <stdint.h>
#include <string.h>
#include "nrf.h"
#include "nordic_common.h"
#include "boards.h"
#include "nrf_delay.h"
#include "nrf_drv_twi.h"
#include "app_error.h"

#include "eMPL_util.h"
#include "inv_mpu.h"
#include "inv_mpu_dmp_motion_driver.h"

#include "nrf_log.h"

#define DMP_FIFO_RATE					 20											/** sample rate for IMU in Hz. */
#define DMP_TAP_COUNT					 2											/** number of taps on IMU to trigger callback */

#if defined(HA_DEV_BOARD)
	#define IMU_INT_PIN_NUMBER	(24U)
#elif defined(HA_PROD_BOARD)
	#define IMU_INT_PIN_NUMBER	(3U)
#elif defined(HA_NRF5x_SHIELD)
#if defined(NRF51)
	#define IMU_INT_PIN_NUMBER	(3U)
#elif defined(NRF52)
	#define IMU_INT_PIN_NUMBER	(28U)
#endif
#else
	#error "Must define HA_DEV_BOARD, HA_PROD_BOARD or HA_NRF5x_SHIELD"
#endif

static volatile bool			 m_is_servicing_dmp = false;				/**< DMP int callback flag. */

static void imu_cb(void)
{
	if (!m_is_servicing_dmp) {
		m_is_servicing_dmp = true;
	}
}

/**@brief Function for IMU initialization.
 */
static void imu_init(void)
{
	uint32_t err_code;
	
	struct int_param_s imu_params = { 0 };
	imu_params.pin = IMU_INT_PIN_NUMBER;
	imu_params.active_low = 1;
	imu_params.cb = imu_cb;

	err_code = mpu_init(&imu_params);
	APP_ERROR_CHECK(err_code);
	
	// Quick read test of WHOAMI value
	unsigned char whoami = 0x00;
	err_code = nrf5x_i2c_read(0x68, 0x75, 1, &whoami);
	if (err_code == NRF_SUCCESS) {
		if (whoami != 0x71) {
			APP_ERROR_HANDLER(0xABAD1DEA);
		}
	}

	err_code = mpu_set_sensors(INV_XYZ_ACCEL);
	APP_ERROR_CHECK(err_code);

	err_code = dmp_load_motion_driver_firmware();
	APP_ERROR_CHECK(err_code);

	err_code = dmp_enable_feature(DMP_FEATURE_SEND_RAW_ACCEL | DMP_FEATURE_TAP);
	APP_ERROR_CHECK(err_code);

	err_code = mpu_set_dmp_state(1);
	APP_ERROR_CHECK(err_code);
}

/**@brief Function for application main entry.
 */
int main(void)
{
    // Initialize.
	imu_init();
}
